#include <lflinkedlist.h>
#include <lfmemory.h>
#include <limits.h>
#include <stdio.h>

#include "lfinput.h"
#include "lfbool.h"

static int part1(List *a) {
    ListNode *n = a->head;

    int inc = LFFALSE;
    if (*(int *) n->data < *(int *) n->next->data) {
        inc = LFTRUE;
    }

    int safe = LFTRUE;
    while (n->next != NULL) {
        int prev = *(int *) n->data, cur = *(int *) n->next->data;
        if (inc == LFTRUE) {
            if (prev > cur || prev == cur || cur - prev > 3) {
                safe = LFFALSE;
                break;
            }
        } else {
            if (prev < cur || prev == cur || prev - cur > 3) {
                safe = LFFALSE;
                break;
            }
        }
        n = n->next;
    }

    return safe;
}

static int part2(List *a) {
    ListNode *n = a->head;

    int inc = LFFALSE;
    if (*(int *) n->data < *(int *) n->next->data) {
        inc = LFTRUE;
    }
    int bad = 0;

    int safe = LFTRUE;
    while (n->next != NULL) {
        const int prev = *(int *) n->data;
        const int cur = *(int *) n->next->data;
        if (inc == LFTRUE) {
            if (prev > cur || prev == cur || cur - prev > 3) {
                if (bad == 0) {
                    void *t;
                    ll_remove_next(a, n, &t);
                    bad = 1;
                    continue;
                }
                safe = LFFALSE;
                break;
            }
        } else {
            if (prev < cur || prev == cur || prev - cur > 3) {
                if (bad == 0) {
                    void *t;
                    ll_remove_next(a, n, &t);
                    bad = 1;
                    continue;
                }
                safe = LFFALSE;
                break;
            }
        }
        n = n->next;
    }

    return safe;
}

void advent2024day02(void) {
    size_t sz = 0;
    char **input = get_lines("input/2024/02", &sz);
    char *errstr = NULL;
    int p1 = 0, p2 = 0;

    for (size_t i = 0; i < sz; i++) {
        List *a = malloc(sizeof(List));
        ll_init(a, NULL);
        ArenaAllocator *arena = malloc(sizeof(ArenaAllocator));
        arena_init(arena, sizeof(int) * 32);

        size_t sp_sz = 0;
        char **sp = split(input[i], &sp_sz, " ");

        for (size_t j = 0; j < sp_sz; j++) {
            int *t = arena_malloc(arena, sizeof(int));
            *t = (int) strtonum(sp[j], INT_MIN, INT_MAX, &errstr);
            if (errstr != NULL) {
                printf("ERR: %s\n", errstr);
                exit(1);
            }
            ll_ins_next(a, a->tail, t);
        }

        p1 += part1(a);
        p2 += part2(a);

        free(sp);
        ll_destroy(a);
        arena_free(arena);
    }

    printf("%d\n", p1);
    printf("%d\n", p2);

    del_lines(input);
}
